package com.rjconsultores.ventaboletos.web.gui.controladores.impressaofiscal;

import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

import javax.sql.DataSource;

import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Controller;
import org.zkoss.zk.ui.Component;
import org.zkoss.zk.ui.Executions;
import org.zkoss.zk.ui.event.Event;
import org.zkoss.zul.Button;
import org.zkoss.zul.Comboitem;
import org.zkoss.zul.Datebox;
import org.zkoss.zul.Filedownload;

import com.rjconsultores.ventaboletos.entidad.Empresa;
import com.rjconsultores.ventaboletos.entidad.Estado;
import com.rjconsultores.ventaboletos.service.EmpresaService;
import com.rjconsultores.ventaboletos.service.EstadoService;
import com.rjconsultores.ventaboletos.service.FiscalService;
import com.rjconsultores.ventaboletos.web.utilerias.MyComboboxEstandar;
import com.rjconsultores.ventaboletos.web.utilerias.MyGenericForwardComposer;

@Controller("busquedaExportacaoFiscalController")
@Scope("prototype")
public class BusquedaExportacaoFiscalController extends MyGenericForwardComposer {
	private static final long serialVersionUID = 1L;
	
	private static Logger log = Logger.getLogger(BusquedaExportacaoFiscalController.class);

	@Autowired
	private FiscalService fiscalService;
	
	@Autowired
	private EmpresaService empresaService;
	
	@Autowired
	private EstadoService estadoService;
	
	private List<Empresa> lsEmpresa;
	private List<Estado> lsEstado;

	private Datebox datInicial;
	private Datebox datFinal;
	
	private MyComboboxEstandar cmbEmpresa;
	private MyComboboxEstandar cmbEstado;
	
	private Button btnExeExportacaoECF;
	private Button btnExeExportacaoRMD;

	@Override
	public void doAfterCompose(Component comp) throws Exception {
		lsEmpresa = empresaService.obtenerTodos();
		lsEstado = estadoService.obtenerTodos();
		
		super.doAfterCompose(comp);

		defineImportacao();
	}

	private void defineImportacao() {
		boolean isEcf = Executions.getCurrent().getArg().get("ECF") == null ? Boolean.FALSE : (Boolean) Executions.getCurrent().getArg().get("ECF");
		if (isEcf) {
			btnExeExportacaoECF.setVisible(true);
		} else {
			btnExeExportacaoECF.setVisible(false);
		}

		boolean isRmd = Executions.getCurrent().getArg().get("RMD") == null ? Boolean.FALSE : (Boolean) Executions.getCurrent().getArg().get("RMD");
		if (isRmd) {
			btnExeExportacaoRMD.setVisible(true);
		} else {
			btnExeExportacaoRMD.setVisible(false);
		}
	}

	public void onClick$btnExeExportacaoECF(Event ev) throws InterruptedException {
		Empresa empresa = null;
		Estado estado = null;
		
		Comboitem itemEmpresa = cmbEmpresa.getSelectedItem();
		Comboitem itemEstado = cmbEstado.getSelectedItem();

		if (itemEmpresa != null) {
			empresa = (Empresa) itemEmpresa.getValue();
		}

		if (itemEstado != null) {
			estado = (Estado) itemEstado.getValue();
		}

		if (empresa != null && estado != null) {
			String nomeArquivo = getNomeArquivoExportacaoECF(datInicial.getValue(), estado, "ECF");
			
			File arquivo = fiscalService.buscarArquivoExportacaoECF(datInicial.getValue(), datFinal.getValue(), empresa, estado, nomeArquivo);

			try {
				downloadArquivo(arquivo, nomeArquivo);
				
			} catch (Exception e) {
				log.error(e.getMessage(), e);
			}
		}
	}

	private String getNomeArquivoExportacaoECF(Date dataInicial, Estado estado, String tipo) {
		String[] mesano = new SimpleDateFormat("MM/yyyy").format(dataInicial).split("/");
		
		String mes = mesano[0].toString(); 
		String ano = mesano[1].toString(); 
		
		String nome = "";
		
		switch (Integer.valueOf(mes)) {
		case 1: nome = "Jan"; break;
		case 2: nome = "Fev"; break;
		case 3: nome = "Mar"; break;
		case 4: nome = "Abr"; break;
		case 5: nome = "Mai"; break;
		case 6: nome = "Jun"; break;
		case 7: nome = "Jul"; break;
		case 8: nome = "Ago"; break;
		case 9: nome = "Set"; break;
		case 10: nome = "Out"; break;
		case 11: nome = "Nov"; break;
		case 12: nome = "Dez"; break;
		default: break;
		}
		
		String nomeArquivo = "TotalBus " + estado.getCveestado() + " " + tipo + " " + nome + ano;

		return nomeArquivo;
	}
	
	public void onClick$btnExeExportacaoRMD(Event ev) throws InterruptedException {
		Empresa empresa = null;
		Estado estado = null;
		
		Comboitem itemEmpresa = cmbEmpresa.getSelectedItem();
		Comboitem itemEstado = cmbEstado.getSelectedItem();

		if (itemEmpresa != null) {
			empresa = (Empresa) itemEmpresa.getValue();
		}

		if (itemEstado != null) {
			estado = (Estado) itemEstado.getValue();
		}

		if (empresa != null && estado != null) {
			String nomeArquivo = getNomeArquivoExportacaoECF(datInicial.getValue(), estado, "BPR");
			
			File arquivo = fiscalService.buscarArquivoExportacaoRMD(datInicial.getValue(), datFinal.getValue(), empresa, estado, nomeArquivo);

			try {
				downloadArquivo(arquivo, nomeArquivo);
				
			} catch (Exception e) {
				log.error(e.getMessage(), e);
			}
		}
	}

	private void downloadArquivo(File arquivoTxt, String nome) throws Exception {
		String contentType = "application/notepad";
		InputStream isFile = new FileInputStream(arquivoTxt);
		String nomeArquivo = arquivoTxt.getName();

		if (nome != null && !nome.isEmpty()) {
			nomeArquivo = nome + ".fs";
		}
		
		Filedownload.save(isFile, contentType, nomeArquivo);

		if (arquivoTxt != null) {
			arquivoTxt.delete();
		}
	}

	public List<Empresa> getLsEmpresa() {
		return lsEmpresa;
	}

	public List<Estado> getLsEstado() {
		return lsEstado;
	}

	public void setLsEstado(List<Estado> lsEstado) {
		this.lsEstado = lsEstado;
	}
}
